/*************************************************
* MPI Algorithms Header File                     *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#ifndef BOTAN_MP_CORE_H__
#define BOTAN_MP_CORE_H__

#include <botan/mp_types.h>

extern "C" {

using Botan::word;
using Botan::u32bit;
using Botan::s32bit;
using Botan::MP_WORD_MAX;
using Botan::MP_WORD_TOP_BIT;

/*************************************************
* Two Argument MP Core                           *
*************************************************/
void bigint_add2(word* const, u32bit, const word*, u32bit);
void bigint_sub2(word* const, u32bit, const word*, u32bit);
void bigint_linmul2(word* const, u32bit, word);

/*************************************************
* Three Argument MP Core                         *
*************************************************/
void bigint_add3(word* const, const word*, u32bit, const word*, u32bit);
void bigint_sub3(word* const , const word*, u32bit, const word*, u32bit);
void bigint_linmul3(word* const, const word*, u32bit, word);

/*************************************************
* MP Shifting                                    *
*************************************************/
void bigint_shl1(word*, u32bit, u32bit, u32bit);
void bigint_shl2(word*, const word*, u32bit, u32bit, u32bit);
void bigint_shr1(word*, u32bit, u32bit, u32bit);
void bigint_shr2(word*, const word*, u32bit, u32bit, u32bit);

/*************************************************
* Other MP Algorithms                            *
*************************************************/
u32bit bigint_divcore(word, word, word, word, word, word);
s32bit bigint_cmp(const word*, u32bit, const word*, u32bit);
u32bit bigint_sigwords(const word*, u32bit);
word bigint_divop(word, word, word);
word bigint_modop(word, word, word);

}

namespace Botan {

/*************************************************
* MP Multiplication                              *
*************************************************/
void bigint_mul3(word* const, u32bit,
                 const word*, u32bit, u32bit,
                 const word*, u32bit, u32bit);

}

#endif
